package sun.management.snmp.jvminstr;

import java.lang.management.ThreadMXBean;
import com.sun.jmx.snmp.SnmpStatusException;
import sun.management.snmp.jvmmib.JVM_MANAGEMENT_MIBOidTable;
import sun.management.snmp.util.MibLogger;
import java.lang.management.ThreadInfo;
import java.io.Serializable;
import sun.management.snmp.jvmmib.JvmThreadInstanceEntryMBean;

public class JvmThreadInstanceEntryImpl implements JvmThreadInstanceEntryMBean, Serializable
{
    static final long serialVersionUID = 910173589985461347L;
    private final ThreadInfo info;
    private final Byte[] index;
    private static String jvmThreadInstIndexOid;
    static final MibLogger log;
    
    public JvmThreadInstanceEntryImpl(ThreadInfo info, Byte[] index) {
        this.info = info;
        this.index = index;
    }
    
    public static String getJvmThreadInstIndexOid() throws SnmpStatusException {
        if (JvmThreadInstanceEntryImpl.jvmThreadInstIndexOid == null) {
            JvmThreadInstanceEntryImpl.jvmThreadInstIndexOid = new JVM_MANAGEMENT_MIBOidTable().resolveVarName("jvmThreadInstIndex").getOid();
        }
        return JvmThreadInstanceEntryImpl.jvmThreadInstIndexOid;
    }
    
    public String getJvmThreadInstLockOwnerPtr() throws SnmpStatusException {
        long lockOwnerId = this.info.getLockOwnerId();
        if (lockOwnerId == -1L) {
            return new String("0.0");
        }
        return getJvmThreadInstIndexOid() + "." + JvmThreadInstanceTableMetaImpl.makeOid(lockOwnerId).toString();
    }
    
    private String validDisplayStringTC(String s) {
        return JVM_MANAGEMENT_MIB_IMPL.validDisplayStringTC(s);
    }
    
    private String validJavaObjectNameTC(String s) {
        return JVM_MANAGEMENT_MIB_IMPL.validJavaObjectNameTC(s);
    }
    
    private String validPathElementTC(String s) {
        return JVM_MANAGEMENT_MIB_IMPL.validPathElementTC(s);
    }
    
    public String getJvmThreadInstLockName() throws SnmpStatusException {
        return this.validJavaObjectNameTC(this.info.getLockName());
    }
    
    public String getJvmThreadInstName() throws SnmpStatusException {
        return this.validJavaObjectNameTC(this.info.getThreadName());
    }
    
    public Long getJvmThreadInstCpuTimeNs() throws SnmpStatusException {
        long threadCpuTime = 0L;
        ThreadMXBean threadMXBean = JvmThreadingImpl.getThreadMXBean();
        try {
            if (threadMXBean.isThreadCpuTimeSupported()) {
                threadCpuTime = threadMXBean.getThreadCpuTime(this.info.getThreadId());
                JvmThreadInstanceEntryImpl.log.debug("getJvmThreadInstCpuTimeNs", "Cpu time ns : " + threadCpuTime);
                if (threadCpuTime == -1L) {
                    threadCpuTime = 0L;
                }
            }
        }
        catch (UnsatisfiedLinkError obj) {
            JvmThreadInstanceEntryImpl.log.debug("getJvmThreadInstCpuTimeNs", "Operation not supported: " + obj);
        }
        return new Long(threadCpuTime);
    }
    
    public Long getJvmThreadInstBlockTimeMs() throws SnmpStatusException {
        long blockedTime = 0L;
        if (JvmThreadingImpl.getThreadMXBean().isThreadContentionMonitoringSupported()) {
            blockedTime = this.info.getBlockedTime();
            if (blockedTime == -1L) {
                blockedTime = 0L;
            }
        }
        return new Long(blockedTime);
    }
    
    public Long getJvmThreadInstBlockCount() throws SnmpStatusException {
        return new Long(this.info.getBlockedCount());
    }
    
    public Long getJvmThreadInstWaitTimeMs() throws SnmpStatusException {
        long waitedTime = 0L;
        if (JvmThreadingImpl.getThreadMXBean().isThreadContentionMonitoringSupported()) {
            waitedTime = this.info.getWaitedTime();
            if (waitedTime == -1L) {
                waitedTime = 0L;
            }
        }
        return new Long(waitedTime);
    }
    
    public Long getJvmThreadInstWaitCount() throws SnmpStatusException {
        return new Long(this.info.getWaitedCount());
    }
    
    public Byte[] getJvmThreadInstState() throws SnmpStatusException {
        return JvmThreadInstanceEntryImpl.ThreadStateMap.getState(this.info);
    }
    
    public Long getJvmThreadInstId() throws SnmpStatusException {
        return new Long(this.info.getThreadId());
    }
    
    public Byte[] getJvmThreadInstIndex() throws SnmpStatusException {
        return this.index;
    }
    
    private String getJvmThreadInstStackTrace() throws SnmpStatusException {
        StackTraceElement[] stackTrace = this.info.getStackTrace();
        StringBuffer sb = new StringBuffer();
        int length = stackTrace.length;
        JvmThreadInstanceEntryImpl.log.debug("getJvmThreadInstStackTrace", "Stack size : " + length);
        for (int i = 0; i < length; ++i) {
            JvmThreadInstanceEntryImpl.log.debug("getJvmThreadInstStackTrace", "Append " + stackTrace[i].toString());
            sb.append(stackTrace[i].toString());
            if (i < length) {
                sb.append("\n");
            }
        }
        return this.validPathElementTC(sb.toString());
    }
    
    static {
        JvmThreadInstanceEntryImpl.jvmThreadInstIndexOid = null;
        log = new MibLogger(JvmThreadInstanceEntryImpl.class);
    }
    
    public static final class ThreadStateMap
    {
        public static final byte mask0 = 63;
        public static final byte mask1 = Byte.MIN_VALUE;
        
        private static void setBit(byte[] array, int n, byte b) {
            array[n] |= b;
        }
        
        public static void setNative(byte[] array) {
            setBit(array, 0, (byte)(-128));
        }
        
        public static void setSuspended(byte[] array) {
            setBit(array, 0, (byte)64);
        }
        
        public static void setState(byte[] array, Thread.State state) {
            switch (state) {
                case BLOCKED: {
                    setBit(array, 0, (byte)8);
                    return;
                }
                case NEW: {
                    setBit(array, 0, (byte)32);
                    return;
                }
                case RUNNABLE: {
                    setBit(array, 0, (byte)16);
                    return;
                }
                case TERMINATED: {
                    setBit(array, 0, (byte)4);
                    return;
                }
                case TIMED_WAITING: {
                    setBit(array, 0, (byte)1);
                    return;
                }
                case WAITING: {
                    setBit(array, 0, (byte)2);
                    return;
                }
                default: {
                    return;
                }
            }
        }
        
        public static void checkOther(byte[] array) {
            if ((array[0] & 0x3F) == 0x0 && (array[1] & 0xFFFFFF80) == 0x0) {
                setBit(array, 1, (byte)(-128));
            }
        }
        
        public static Byte[] getState(ThreadInfo threadInfo) {
            byte[] array = { 0, 0 };
            try {
                Thread.State threadState = threadInfo.getThreadState();
                int inNative = threadInfo.isInNative();
                int suspended = threadInfo.isSuspended();
                JvmThreadInstanceEntryImpl.log.debug("getJvmThreadInstState", "[State=" + threadState + ",isInNative=" + (boolean)inNative + ",isSuspended=" + (boolean)suspended + "]");
                setState(array, threadState);
                if (!(inNative == 0)) {
                    setNative(array);
                }
                if (!(suspended == 0)) {
                    setSuspended(array);
                }
                checkOther(array);
            }
            catch (RuntimeException obj) {
                array[0] = 0;
                array[1] = -128;
                JvmThreadInstanceEntryImpl.log.trace("getJvmThreadInstState", "Unexpected exception: " + obj);
                JvmThreadInstanceEntryImpl.log.debug("getJvmThreadInstState", obj);
            }
            return new Byte[] { new Byte(array[0]), new Byte(array[1]) };
        }
        
        public static final class Byte0
        {
            public static final byte inNative = Byte.MIN_VALUE;
            public static final byte suspended = 64;
            public static final byte newThread = 32;
            public static final byte runnable = 16;
            public static final byte blocked = 8;
            public static final byte terminated = 4;
            public static final byte waiting = 2;
            public static final byte timedWaiting = 1;
        }
        
        public static final class Byte1
        {
            public static final byte other = Byte.MIN_VALUE;
            public static final byte reserved10 = 64;
            public static final byte reserved11 = 32;
            public static final byte reserved12 = 16;
            public static final byte reserved13 = 8;
            public static final byte reserved14 = 4;
            public static final byte reserved15 = 2;
            public static final byte reserved16 = 1;
        }
    }
}
